/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2010 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */
package com.openexchange.usm.contenttypes.tasks.impl;

import static com.openexchange.usm.api.datatypes.DataTypes.*;
import java.io.Serializable;
import com.openexchange.usm.api.contenttypes.ContentTypeField;
import com.openexchange.usm.api.contenttypes.ContentTypeTransferHandler;
import com.openexchange.usm.api.contenttypes.DefaultContentTypes;
import com.openexchange.usm.api.contenttypes.TaskContentType;
import com.openexchange.usm.api.datatypes.PIMAttachmentsDataType;
import com.openexchange.usm.api.exceptions.AuthenticationFailedException;
import com.openexchange.usm.api.exceptions.DeserializationFailedException;
import com.openexchange.usm.api.exceptions.OXCommunicationException;
import com.openexchange.usm.api.session.DataObject;
import com.openexchange.usm.contenttypes.util.AbstractContentType;
import com.openexchange.usm.contenttypes.util.UtilConstants;
import com.openexchange.usm.datatypes.tasks.calendar.CommonCalendarTasksFieldNames;
import com.openexchange.usm.datatypes.tasks.calendar.ParticipantIdentifierDataType;
import com.openexchange.usm.datatypes.tasks.calendar.UserParticipantObjectDataType;

/**
 * Defines the Tasks content type.
 * @author ldo
 *
 */
public class TaskContentTypeImpl extends AbstractContentType implements TaskContentType {

	private static final ContentTypeField[] FIELDS = {
	//Common object data
			new ContentTypeField(1, UtilConstants.FIELD_ID, STRING, false, true, false, false), //Object ID
			new ContentTypeField(20, UtilConstants.FOLDER_ID, STRING, false, false, true, false), //Object ID of the parent folder.
			new ContentTypeField(2, UtilConstants.CREATED_BY, STRING), //User ID of the user who created this object.
			new ContentTypeField(3, UtilConstants.MODIFIED_BY, STRING), //User ID of the user who last modified this object.
			new ContentTypeField(4, UtilConstants.CREATION_DATE, TIME), //Date and time of creation.
			new ContentTypeField(5, UtilConstants.LAST_MODIFIED, TIME), //Date and time of the last modification.
			new ContentTypeField(100, UtilConstants.CATEGORIES, STRING), // Each element is a string, naming a category. Order is preserved. Changing the order counts as modification of the object.
			new ContentTypeField(101, UtilConstants.PRIVATE_FLAG, BOOLEAN), //Overrides folder permissions in shared private folders: When true, this object is not visible to anyone except the owner.
			new ContentTypeField(104, UtilConstants.NUMBER_OF_ATTACHMENTS, NUMBER), //Number of attachments
			//Detailed task and appointment data
			new ContentTypeField(200, CommonCalendarTasksFieldNames.TITLE, STRING), //Short description.
			new ContentTypeField(201, CommonCalendarTasksFieldNames.START_DATE, TIME), //Inclusive start of the event as Date for tasks and whole day appointments and Time for normal appointments
			new ContentTypeField(202, CommonCalendarTasksFieldNames.END_DATE, TIME), //Exclusive end of the event as Date for tasks and whole day appointments and as Time for normal appointments
			new ContentTypeField(203, CommonCalendarTasksFieldNames.NOTE, STRING), //Long description
			new ContentTypeField(204, CommonCalendarTasksFieldNames.ALARM, TIME), //Specifies when to notify the participants as the number of minutes before the end of the task or before the start of the appointment
			new ContentTypeField(207, CommonCalendarTasksFieldNames.RECURRENCE_POSITION, NUMBER), //1-based position of an individual appointment in a sequence. Present if and only if recurrence_type > 0
			new ContentTypeField(208, CommonCalendarTasksFieldNames.RECURRENCE_DATE_POSITION, DATE), //Date of an individual appointment in a sequence. Present if and only if recurrence_type > 0.
			new ContentTypeField(209, CommonCalendarTasksFieldNames.RECURRENCE_TYPE, NUMBER_NOT_NULL), //Specifies the type of the recurrence for a task sequence: 0 -none(single event); 1-daily; 2-weekly;3-monthly; 4-yearly;
			new ContentTypeField(212, CommonCalendarTasksFieldNames.DAYS, NUMBER), //Specifies which days of the week are part of a sequence. The value is a bit field with bit 0 indicating sunday, bit 1 indicating monday and so on
			new ContentTypeField(213, CommonCalendarTasksFieldNames.DAY_IN_MONTH, NUMBER), //Specifies which day of a month is part of the sequence. Counting starts with 1
			new ContentTypeField(214, CommonCalendarTasksFieldNames.MONTH, NUMBER), //Month of the year in yearly sequences. 0 represents January, 1 represents February and so on
			new ContentTypeField(215, CommonCalendarTasksFieldNames.INTERVAL, NUMBER), //Specifies an integer multiplier to the interval specified by recurrence_type. Present if and only if recurrence_type > 0
			new ContentTypeField(216, CommonCalendarTasksFieldNames.UNTIL, DATE), //Exclusive end date of a sequence. May be present only if recurrence_type > 0
			new ContentTypeField(217, CommonCalendarTasksFieldNames.NOTIFICATION, BOOLEAN), // If true, all participants are notified of any changes to this object. This flag is valid for the current change only, i. e. it is not stored in the database and is never sent by the server to the client
			new ContentTypeField(220, CommonCalendarTasksFieldNames.PARTICIPANTS, new ParticipantIdentifierDataType(),
					true, false), //Each element identifies a participant, user group or booked resource
			new ContentTypeField(221, CommonCalendarTasksFieldNames.USERS, new UserParticipantObjectDataType(), true, false), //Each element represents a participant. User groups are resolved and are represented by their members. Any user can occur only once.
			new ContentTypeField(222, CommonCalendarTasksFieldNames.OCCURENCES, NUMBER), //Specifies how often a recurrence should appear. May be present only if recurrence_type > 0
			//Detailed task data
			new ContentTypeField(300, TaskConstants.STATUS, NUMBER), //Status of the task:	1 not started	2 	in progress	3 	done 4 	waiting	 5 	deferred
			new ContentTypeField(301, TaskConstants.PERCENT_COMPLETED, NUMBER), //How much of the task is completed. An integer number between 0 and 100.
			new ContentTypeField(302, TaskConstants.ACTUAL_COSTS, STRING), //XXX: Type is missing in the HTTP_API docs
			new ContentTypeField(303, TaskConstants.ACTUAL_DURATION, STRING), //XXX: Type is missing in the HTTP_API docs
			new ContentTypeField(305, TaskConstants.BILLING_INFORMATION, STRING), //XXX: Type is missing in the HTTP_API docs
			new ContentTypeField(307, TaskConstants.TARGET_COSTS, STRING), //XXX: Type is missing in the HTTP_API docs
			new ContentTypeField(308, TaskConstants.TARGET_DURATION, STRING), //XXX: Type is missing in the HTTP_API docs
			new ContentTypeField(309, TaskConstants.PRIORITY, NUMBER), //1 = LOW, 2 = MEDIUM, 3 = HIGH
			new ContentTypeField(312, TaskConstants.CURRENCY, STRING), //XXX: Type is missing in the HTTP_API docs
			new ContentTypeField(313, TaskConstants.TRIP_METER, STRING), //XXX: Type is missing in the HTTP_API docs
			new ContentTypeField(314, TaskConstants.COMPANIES, STRING), //XXX: Type is missing in the HTTP_API docs
			new ContentTypeField(315, TaskConstants.DATE_COMPLETED, DATE), //XXX: Type is missing in the HTTP_API docs
			new ContentTypeField(102, TaskConstants.COLOR_LABEL, NUMBER), // Number from 1-10
			new ContentTypeField(105, UtilConstants.ATTACHMENTS_LAST_MODIFIED, new PIMAttachmentsDataType()), // New complex field that contains attachments_last_modified for updates and additionally an array PIMAttachment for persistent storing in the SyncState
			new ContentTypeField(223, CommonCalendarTasksFieldNames.UID, STRING), //the UID of the task
	};

	private TaskContentTypeTransferHandlerImpl _transferHandler;

	/**
	 * Constructor is package private to discourage access from outside the bundle (but to allow usage in test cases)
	 */
	TaskContentTypeImpl() {
	}

	public ContentTypeField[] getFields() {
		return FIELDS;
	}

	public String getID() {
		return DefaultContentTypes.TASK_ID;
	}

	@Override
	public String toString() {
		return getID();
	}

	public ContentTypeTransferHandler getTransferHandler() {
		return _transferHandler;
	}

	public void setTransferHandler(TaskContentTypeTransferHandlerImpl transferHandler) {
		_transferHandler = transferHandler;
	}

	@Override
	public int hashCode() {
		return 54931;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (!(obj instanceof TaskContentType))
			return false;
		return true;
	}

	public void confirm(DataObject object, int response) throws AuthenticationFailedException, OXCommunicationException {
		_transferHandler.confirm(object, response);
	}

	public void confirm(DataObject object, int response, String confirmMessage) throws AuthenticationFailedException,
			OXCommunicationException {
		_transferHandler.confirm(object, response, confirmMessage);
	}

	public int getCode() {
		return DefaultContentTypes.TASK_CODE;
	}

	public boolean supportsPIMAttachments() {
		return true;
	}

	@Override
	public void deserializeData(Serializable[] data, int offset, Object[] fieldContent)
			throws DeserializationFailedException {
		// Special code to handle removal of project_id field, can be removed once no old version is in use any more
		if (data.length == 41 + offset && fieldContent.length == 40) {
			Serializable[] newData = new Serializable[40 + offset];
			for (int i = 0; i < 31 + offset; i++)
				newData[i] = data[i];
			for (int i = 31 + offset; i < 40 + offset; i++)
				newData[i] = data[i + 1];
			data = newData;
		}
		super.deserializeData(data, offset, fieldContent);
	}

	public Object getObjectGroupOwner(DataObject object) {
		return object.getID();
	}
}
